//
Double_t Schiff(Double_t Z, Double_t T, Double_t k){
  
  // L. I. Schiff, Physical Review 83, 252 (1951).
  
  Double_t me = 0.511; // Electron mass (MeV)
  Double_t E = T + me; // Electron energy
  Double_t Ep = E-k; // Scattered electron energy (MeV)
  
  Double_t alpha = 1.0/137.036; // Fine-structure Constant
  Double_t re = 2.818e-13; // classical electron radius (cm)
  
  Double_t C = 111.0; 
  Double_t M = 1.0 / ( (me*k/(2.0*E*Ep))**2 + (Z**1/3/C)**2 );
  Double_t b = 2.0*E*Ep*Z**1/2 / (C*me*k);
  Double_t atan = TMath::ATan(b);
  
  //
  // 
  
  Double_t schiff = 2.0*Z**2*re**2*alpha/k * ( ((E**2+Ep**2)/E**2 - 2.0*Ep/(3.0*E)) * (TMath::Log(M) + 1.0 - 2.0/b*atan) + Ep/E * ( 2.0/b**2*TMath::Log(1.0+b**2) + 4.0*(2.0-b**2)/(3.0*b**3)*atan - 8.0/(3.0*b**2)+2.0/9.0)); //  cm2/MeV
  
  //  cout << "E = " << E << " k = " << k << " Schiff = " << schiff <<endl;
  
  return schiff; //  cm2/MeV
  
}
//
Double_t Ng(Double_t T, Double_t k, Double_t Delta){
  
  if (k>T) return 0.0;
  
  Int_t w = 100;
  Double_t Z = 29.0; // Cu
  Double_t ng = 0.0;
  Double_t DE = Delta/w;
  
  //
  
  for (Int_t i=0; i<w; i++){ 
    ng = ng + Schiff( Z,T, k-Delta + (i+1)*DE )*DE;
  }
  
  return ng; // cm2
  
}
//

void g17On_rate_ThrCut(){
  // g + 17O -> n + 16O
  // Rate versus electron beam kinetic energy

  gROOT->Reset();
  
  // O17 Neutron Cross Section (mb)
  gROOT->ProcessLine(".L csO17gn_interpolation.C++");
  //  cout << " Cross Section (mb) = " << csO17gn_interpolation(6.0) << endl;
  ////
  
#define MAX_nentries 436;
  
  TString pstit;
  
  // define style here 
  // general parameters
  gStyle->SetOptDate(0);     gStyle->SetOptTitle(0);
  gStyle->SetStatColor(10);  gStyle->SetStatH(0.2);
  gStyle->SetStatW(0.3);     gStyle->SetOptStat(0); 
  
  // canvas parameters
  gStyle->SetFrameBorderMode(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetFrameFillColor(0);
  gStyle->SetCanvasColor(0);
  
  //
  
  // pads parameters
  //  gStyle->SetPadColor(39); 
  gStyle->SetPadColor(0); 
  gStyle->SetPadBorderMode(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetPadBottomMargin(0.18);
  gStyle->SetPadRightMargin(0.10);
  gStyle->SetPadLeftMargin(0.14);
  gStyle->SetLabelSize(0.04,"x");
  gStyle->SetLabelSize(0.04,"y");
  gStyle->SetTitleXSize(0.08);
  gStyle->SetPaperSize(10,12);
  
  gStyle->SetOptLogx(0);
  gStyle->SetOptLogy(1);
  
  // gStyle->SetNdivisions(505);
  
  gStyle->SetTitleYOffset(0.8);
  gStyle->SetTitleYSize(0.10);
  gROOT->ForceStyle();  

  //// Schiff Spectrum
  
  Double_t Nv = 6.02214179e+23; // Avogadro constant (/mol)  
  
  Double_t L = 3.0; // cm, Chamber length
  Double_t rho = 0.846; // gm/cm^3, N2O density
  Double_t NMol = L*rho/44.0; // Number of N2O moles (/cm^2)
  Double_t NO = Nv*NMol; // Number of Oxygen nuclei (/cm^2)
  Double_t NN14 = 2.0*Nv*NMol; // Number of N nuclei (/cm^2)
  //
  
  Double_t NO16 = NO * 0.9976 / 1.0; // Number of Oxygen 16 nuclei
  
  Double_t Depletion = 1.0; // Depletion of Oxygen isotopes
  Double_t NO17 = NO * 0.00038 / Depletion; // Number of Oxygen 17 nuclei
  Double_t NO18 = NO * 0.00205 / Depletion; // Number of Oxygen 18 nuclei
  
  //
  
  Double_t BremSolidAngle = 6.0; // To find gammas hitting glass cell. 
                                 // Factor to reduce Schiff (with 0.02 mm) to GEANT4 simulation (with 6 mm)

  //
  
  Double_t RadThickness = 0.002; // cm, 0.02 mm Radiator
  Double_t rhoCu = 8.94; // g/cm3 Cu
  Double_t MCu = 63.546; // g/mol Cu (A)
  Double_t Z = 29.0; // Cu
  
  Double_t fCu = rhoCu*RadThickness*Nv/MCu; // Number of Cu atoms/cm^2
  Double_t Ib = 10.0; // Beam current (uA)
  Double_t e = 1.602e-19; // C
  Double_t Ne = 1.0e-6/e; // Number of electrons per 1 uC
  
  Double_t Q17On = 4.1430; // Q-Value of 17O(g,n)16O  
  Double_t Delta = 0.01; // 0.01 MeV bins 
  
  Double_t Neutron_KE[MAX_nentries];
  Double_t weight[MAX_nentries];
  
  Double_t T0[MAX_nentries]; // Electron beam kinetic energy (MeV)
  Double_t err1[MAX_nentries] = 0.0;
  Double_t cs17Ogn;
  Double_t rate17Ogn[MAX_nentries] = 0.0;
 
  Double_t ThrCut = 130.0; // Threshold energy cut of ion 16O in keV
  
  //
  
  if (!gROOT->GetClass("TGenPhaseSpace")) gSystem.Load("libPhysics");
  
  Double_t M17O = 17.0 * 931.494061 - 0.809; // MeV, mass of 17O nucleus
  TLorentzVector target(0.0, 0.0, 0.0, M17O);
  
  Double_t NeutronMass = 1.0*931.494061 + 8.071; // MeV 
  Double_t O16Mass = 16.0 * 931.494061 - 4.737; // MeV 
  
  Double_t masses[2] = {NeutronMass, O16Mass} ; 
  //
   
  //// Generate Phase Space
  
  for (Int_t i=0; i<MAX_nentries; i++) {
    T0[i] = Q17On + (i+1)*Delta; // Electron beam kinetic energy (MeV)
    
    TH1F* h2  = new TH1F("h2","^{16}O Energy", 100,0,1000);
    
    for (Int_t j=0; j<i; j++) {
      
      Double_t Eg = Q17On + j*Delta;
      
      Double_t Ngamma = Ng(T0[i],Eg,Delta) * fCu * Ne;
      //     cout << "E Gamma (MeV) = " << Eg << endl;
      
      Double_t csO17 = csO17gn_interpolation(Eg)*4.0*TMath::Pi()*1.0e-27;
      // mb to cm^2 and from dsigma/dOmega to total cross section
      
      weight[j] = Ngamma * csO17 * NO17;
      
      //    cout << "E Gamma = " << Eg << " Weight = " << weight << endl;
      //
      
      TLorentzVector beam(0.0, 0.0, Eg, Eg);
      TLorentzVector W = beam + target;
      
      TGenPhaseSpace event;
      event.SetDecay(W, 2, masses);
      
      //
      
      Double_t weight1 = event.Generate();
      //    cout << "Weight1 = " << weight1 << endl;
      
      TLorentzVector *pN = event.GetDecay(0);
      TLorentzVector *pO16 = event.GetDecay(1);
      
      //
      
      Neutron_KE[j] = pN->E() - masses[0]; // MeV
      Double_t T16O = (pO16->E() - masses[1]) * 1000.0; // keV
      
      if (T16O>ThrCut) h2->Fill(T16O, weight[j]); // Threshold Cut
    }
    
    Double_t scale = Ib / BremSolidAngle; // Scale to per uA per s
    h2->Scale(scale);
    
    /////////
    
    rate17Ogn[i] = h2->GetSum();
    h2->Delete();
    
  }
  
  /////
  
  TCanvas *g1 = new TCanvas("g1","N2O Rate",40,40,700,500);

  TGraphErrors *grO17gn = new TGraphErrors(MAX_nentries,T0,rate17Ogn,err1,err1);
  g1->SetGrid();
  
  grO17gn->SetMinimum(1.0e-03);
  grO17gn->SetMaximum(1.0e+03);
  grO17gn->GetXaxis()->SetLimits(4.0,8.5);
  
  grO17gn->GetXaxis()->CenterTitle();
  grO17gn->GetXaxis()->SetTitleOffset(1.2);
  grO17gn->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Electron Kinetic Energy (MeV)";
  grO17gn->GetXaxis()->SetTitle(tit.Data());
  
  grO17gn->GetYaxis()->CenterTitle();
  grO17gn->GetYaxis()->SetTitleOffset(1.0);
  grO17gn->GetYaxis()->SetTitleSize(0.06);
  TString tit ="Rate (Hz), 10 #muA";
  grO17gn->GetYaxis()->SetTitle(tit.Data());
  
  //
  grO17gn->SetLineColor(kCyan);
  grO17gn->SetLineWidth(2.0);
  grO17gn->SetLineStyle(1);
  grO17gn->Draw("A C");
  //
  leg = new TLegend(0.20,0.75,0.45,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->AddEntry(grO17gn," ^{17}O(#gamma,n)^{16}O ","L");
  leg->Draw();
  // leg->SetHeader("I_{b} = 10 #muA");
  leg->Draw();
  //
  
  g1->cd();
  pstit = "g17On_rate_ThrCut";
  pstit += ".gif";
  g1->Print(pstit.Data());
  ////
}
