//
Double_t Schiff(Double_t Z, Double_t T, Double_t k){
  
  // L. I. Schiff, Physical Review 83, 252 (1951).
  
  Double_t me = 0.511; // Electron mass (MeV)
  Double_t E = T + me; // Electron energy
  Double_t Ep = E-k; // Scattered electron energy (MeV)
  
  Double_t alpha = 1.0/137.036; // Fine-structure Constant
  Double_t re = 2.818e-13; // classical electron radius (cm)
  
  Double_t C = 111.0; 
  Double_t M = 1.0 / ( (me*k/(2.0*E*Ep))**2 + (Z**1/3/C)**2 );
  Double_t b = 2.0*E*Ep*Z**1/2 / (C*me*k);
  Double_t atan = TMath::ATan(b);
  
  //
  // 
  
  Double_t schiff = 2.0*Z**2*re**2*alpha/k * ( ((E**2+Ep**2)/E**2 - 2.0*Ep/(3.0*E)) * (TMath::Log(M) + 1.0 - 2.0/b*atan) + Ep/E * ( 2.0/b**2*TMath::Log(1.0+b**2) + 4.0*(2.0-b**2)/(3.0*b**3)*atan - 8.0/(3.0*b**2)+2.0/9.0)); //  cm2/MeV
  
  //  cout << "E = " << E << " k = " << k << " Schiff = " << schiff <<endl;
  
  return schiff; //  cm2/MeV
  
}
//
Double_t Ng(Double_t T, Double_t k, Double_t Delta){
  
  if (k>T) return 0.0;
  
  Int_t w = 100;
  Double_t Z = 29.0; // Cu
  Double_t ng = 0.0;
  Double_t DE = Delta/w;
  
  //
  
  for (Int_t i=0; i<w; i++){ 
    ng = ng + Schiff( Z,T, k-Delta + (i+1)*DE )*DE;
  }
  
  return ng; // cm2
  
}
//

void gNeutron(){
  // g + 17O -> n + 16O
  //            n + 16O -> n + 16O
  //            n + 14N -> n + 14N
  //
  // Electron beam kinetic energy (MeV) = 8.0

  gROOT->Reset();
  
  // O17 Neutron Cross Section (mb)
  gROOT->ProcessLine(".L csO17gn_interpolation.C++");
  //  cout << " Cross Section (mb) = " << csO17gn_interpolation(6.0) << endl;
  ////
  
  // N14 Neutron Cross Section (b)
  gROOT->ProcessLine(".L csN14n_interpolation.C++");
  //  cout << " Cross Section (b) = " << csN14n_interpolation(2.0) << endl;
  ////
  
  // O16 Neutron Cross Section (b)
  gROOT->ProcessLine(".L csO16n_interpolation.C++");
  //  cout << " Cross Section (b) = " << csO16n_interpolation(2.0) << endl;
  ////
  
#define MAX_nentries 3857;
  
  TString pstit;
  
  // define style here 
  // general parameters
  gStyle->SetOptDate(0);     gStyle->SetOptTitle(0);
  gStyle->SetStatColor(10);  gStyle->SetStatH(0.2);
  gStyle->SetStatW(0.3);     gStyle->SetOptStat(0); 
  
  // canvas parameters
  gStyle->SetFrameBorderMode(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetFrameFillColor(0);
  gStyle->SetCanvasColor(0);
  
  //
  
  // pads parameters
  //  gStyle->SetPadColor(39); 
  gStyle->SetPadColor(0); 
  gStyle->SetPadBorderMode(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetPadBottomMargin(0.18);
  gStyle->SetPadRightMargin(0.10);
  gStyle->SetPadLeftMargin(0.14);
  gStyle->SetLabelSize(0.04,"x");
  gStyle->SetLabelSize(0.04,"y");
  gStyle->SetTitleXSize(0.08);
  gStyle->SetPaperSize(10,12);
  
  gStyle->SetOptLogx(0);
  gStyle->SetOptLogy(1);
  
  // gStyle->SetNdivisions(505);
  
  gStyle->SetTitleYOffset(0.8);
  gStyle->SetTitleYSize(0.10);
  gROOT->ForceStyle();  

  //// Schiff Spectrum
  
  Double_t Nv = 6.02214179e+23; // Avogadro constant (/mol)  
  
  Double_t L = 3.0; // cm, Chamber length
  Double_t rho = 0.846; // gm/cm^3, N2O density
  Double_t NMol = L*rho/44.0; // Number of N2O moles (/cm^2)
  Double_t NO = Nv*NMol; // Number of Oxygen nuclei (/cm^2)
  Double_t NN14 = 2.0*Nv*NMol; // Number of N nuclei (/cm^2)
  //
  
  Double_t NO16 = NO * 0.9976 / 1.0; // Number of Oxygen 16 nuclei
  
  Double_t Depletion = 1.0; // Depletion of Oxygen isotopes
  Double_t NO17 = NO * 0.00038 / Depletion; // Number of Oxygen 17 nuclei
  Double_t NO18 = NO * 0.00205 / Depletion; // Number of Oxygen 18 nuclei
  
  //
  
  Double_t BremSolidAngle = 6.0; // To find gammas hitting glass cell. 
                                 // Factor to reduce Schiff (with 0.02 mm) to GEANT4 simulation (with 6 mm)

  //
  
  Double_t RadThickness = 0.002; // cm, 0.02 mm Radiator
  Double_t rhoCu = 8.94; // g/cm3 Cu
  Double_t MCu = 63.546; // g/mol Cu (A)
  Double_t Z = 29.0; // Cu
  
  Double_t fCu = rhoCu*RadThickness*Nv/MCu; // Number of Cu atoms/cm^2
  Double_t Ib = 10.0; // Beam current (uA)
  Double_t e = 1.602e-19; // C
  Double_t Ne = 1.0e-6/e; // Number of electrons per 1 uC
  
  Double_t T0 = 8.0; // Electron beam kinetic energy (MeV)
  
  Double_t Q17On = 4.1430; // Q-Value of 17O(g,n)16O
  
  Double_t Delta = 0.001; // 0.001 MeV bins 
  
  Double_t Neutron_KE[MAX_nentries];
  Double_t weight[MAX_nentries];
  
  //
  //
  
  //
  
  if (!gROOT->GetClass("TGenPhaseSpace")) gSystem.Load("libPhysics");
  
  Double_t M17O = 17.0 * 931.494061 - 0.809; // MeV, mass of 17O nucleus
  TLorentzVector target(0.0, 0.0, 0.0, M17O);
  
  Double_t NeutronMass = 1.0*931.494061 + 8.071; // MeV 
  Double_t O16Mass = 16.0 * 931.494061 - 4.737; // MeV 
  
  Double_t masses[2] = {NeutronMass, O16Mass} ; 
  //
   
  TH1F* h1  = new TH1F("h1","Neutron Energy", 500,0,5.0);
  TH1F* h2  = new TH1F("h2","^{16}O Energy", 100,0,1000);
   
  //// Generate Phase Space
  
  for (Int_t i=0; i<MAX_nentries; i++) {
    
    Double_t Eg = Q17On + (i+1)*Delta;
    
    Double_t Ngamma = Ng(T0,Eg,Delta) * fCu * Ne;
    //     cout << " E Gamma (MeV) =" << Eg << endl;
    
    Double_t csO17 = csO17gn_interpolation(Eg)*4.0*TMath::Pi()*1.0e-27;
    // mb to cm^2 and from dsigma/dOmega to total cross section
    
    weight[i] = Ngamma * csO17 * NO17;
    
    //    cout << "E Gamma = " << Eg << " Weight = " << weight << endl;
    //
     
    TLorentzVector beam(0.0, 0.0, Eg, Eg);
    TLorentzVector W = beam + target;
    
    TGenPhaseSpace event;
    event.SetDecay(W, 2, masses);
    
    //
    
    Double_t weight1 = event.Generate();
    //    cout << "Weight1 = " << weight1 << endl;
    
    TLorentzVector *pN = event.GetDecay(0);
    TLorentzVector *pO16 = event.GetDecay(1);
    
    //
    
    Neutron_KE[i] = pN->E() - masses[0]; // MeV
    Double_t T16O = (pO16->E() - masses[1]) * 1000.0; // keV
    
    h1->Fill(Neutron_KE[i], weight[i]);
    h2->Fill(T16O, weight[i]);
  }
  
  Double_t scale = Ib / BremSolidAngle; // Scale to per uA per s
  h1->Scale(scale);
  h2->Scale(scale);
  
  /////////
  ///////// n + 16O -> n + 16O
  
  Double_t O16Mass = 16.0*931.494061 - 4.737; // MeV 
  TLorentzVector target(0.0, 0.0, 0.0, O16Mass);
  
  Double_t NeutronMass = 1.0*931.494061 + 8.071; // MeV 
  
  Double_t masses[2] = {NeutronMass, O16Mass} ; 
  //
  
  TH1F* h3  = new TH1F("h3","Neutron Energy", 500,0,5.0);
  TH1F* h4  = new TH1F("h4","^{16}O Energy", 100,0,1000);
  TH2F* h5  = new TH2F("h5","^{16}O Energy vs Neutron Energy", 500,0,5.0,100,0,1.0);
  //// Generate Phase Space
  
  for (Int_t i=0; i<MAX_nentries; i++) {
    
    Double_t Tn = Neutron_KE[i]; // Neutron kinetic energy (MeV)
    Double_t En = Tn + NeutronMass; // Neutron energy (MeV)
    Double_t NeutronMom = TMath::Sqrt(En**2 - NeutronMass**2);// Neutron momentum (MeV)
    
    TLorentzVector beam(0.0, 0.0, NeutronMom, En);
    TLorentzVector W = beam + target;
    
    TGenPhaseSpace event;
    event.SetDecay(W, 2, masses);
    
    //
    
    Double_t weight1 = event.Generate();
    
    TLorentzVector *pNeutron = event.GetDecay(0);
    TLorentzVector *pO16 = event.GetDecay(1);
    
    //
    
    Double_t TN = pNeutron->E() - masses[0]; // MeV
    Double_t T16O =  (pO16->E() - masses[1]) * 1000.0; // keV
    
    Double_t csnO16n = csO16n_interpolation(Tn)*1.0e-24; // b to cm^2
    
    Double_t  weight2 = csnO16n * NO16 * weight[i];
    
    //
    
    h3->Fill(TN, weight2);
    h4->Fill(T16O, weight2);
    T16O = T16O / 1000.0; // MeV
    
    h5->Fill(TN, T16O, weight2);
    
  }
  
  Double_t scale = Ib / BremSolidAngle; // Scale to per uA per s
  h3->Scale(scale);
  h4->Scale(scale);
  h5->Scale(scale);
  
  /////////
  ///////// n + 14N -> n + 14N
  
  Double_t N14Mass = 14.0*931.494061 + 2.863; // MeV 
  TLorentzVector target(0.0, 0.0, 0.0, N14Mass);
   
  Double_t NeutronMass = 1.0*931.494061 + 8.071; // MeV 
  
  Double_t masses[2] = {NeutronMass, N14Mass} ; 
  //
  
  TH1F* h6  = new TH1F("h6","Neutron Energy", 500,0,5.0);
  TH1F* h7  = new TH1F("h7","^{14}N Energy", 100,0,1000);
  TH2F* h8  = new TH2F("h8","^{14}N Energy vs Neutron Energy", 500,0,5.0,100,0,1.0);
  //// Generate Phase Space
  
  for (Int_t i=0; i<MAX_nentries; i++) {
    
    Double_t Tn = Neutron_KE[i]; // Neutron kinetic energy (MeV)
    Double_t En = Tn + NeutronMass; // Neutron energy (MeV)
    Double_t NeutronMom = TMath::Sqrt(En**2 - NeutronMass**2);// Neutron momentum (MeV)
    
    TLorentzVector beam(0.0, 0.0, NeutronMom, En);
    TLorentzVector W = beam + target;
     
    TGenPhaseSpace event;
    event.SetDecay(W, 2, masses);
    
    //
    
    Double_t weight1 = event.Generate();
    
    TLorentzVector *pNeutron = event.GetDecay(0);
    TLorentzVector *pN14 = event.GetDecay(1);
    
     //
    
    Double_t TN = pNeutron->E() - masses[0]; // MeV
    Double_t T14N =  (pN14->E() - masses[1]) * 1000.0; // keV
    
    Double_t csnN14n = csN14n_interpolation(Tn)*1.0e-24; // b to cm^2
    
    Double_t  weight3 = csnN14n * NN14 * weight[i];
    
    //
    
    h6->Fill(TN, weight3);
    h7->Fill(T14N, weight3);
    T14N = T14N / 1000.0; // MeV
    
    h8->Fill(TN, T14N, weight3);
    
  }
  
  Double_t scale = Ib / BremSolidAngle;  // Scale to per uA per s
  h6->Scale(scale);
  h7->Scale(scale);
  h8->Scale(scale);
  
  /////////
  
  TCanvas *g1 = new TCanvas("g1","Neutron Energy",40,40,700,500);
  
  g1->SetGrid();
  
  h1->SetMinimum(1.0e-06);
  h1->SetMaximum(1.0e+00);
  
  h1->GetXaxis()->CenterTitle();
  h1->GetXaxis()->SetTitleOffset(1.2);
  h1->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Neutron Energy (MeV)";
  h1->GetXaxis()->SetTitle(tit.Data());
  
  h1->GetYaxis()->CenterTitle();
  h1->GetYaxis()->SetTitleOffset(1.0);
  h1->GetYaxis()->SetTitleSize(0.06);
  TString tit ="Neutrons /(10 #muA s 0.01 MeV)";
  h1->GetYaxis()->SetTitle(tit.Data());
  
  h1->SetLineColor(kRed);
  h1->SetLineWidth(2.0);
  h1->SetLineStyle(2);
  h1->Draw("");
  //
  leg = new TLegend(0.55,0.70,0.85,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->AddEntry(h1,"Neutron","L");
  leg->Draw();
  leg->SetHeader("T_{e} = 8.0 MeV");
  leg->Draw();
  //
  
  g1->cd();
  pstit = "gNeutron_Neutron";
  pstit += ".gif";
  g1->Print(pstit.Data());
  
  ////
  //
  
  TCanvas *g2 = new TCanvas("g2","Oxygen 16 Energy",40,40,700,500);
  
  g2->SetGrid();
  
  h2->SetMinimum(1.0e-06);
  h2->SetMaximum(1.0e+00);
  
  h2->GetXaxis()->CenterTitle();
  h2->GetXaxis()->SetTitleOffset(1.2);
  h2->GetXaxis()->SetTitleSize(0.06);
  TString tit = "^{16}O Energy (keV)";
  h2->GetXaxis()->SetTitle(tit.Data());
  
  h2->GetYaxis()->CenterTitle();
  h2->GetYaxis()->SetTitleOffset(1.0);
  h2->GetYaxis()->SetTitleSize(0.06);
  TString tit ="^{16}O /(10 #muA s 10 keV)";
  h2->GetYaxis()->SetTitle(tit.Data());
  
  h2->SetLineColor(kRed);
  h2->SetLineWidth(2.0);
  h2->SetLineStyle(2);
  h2->Draw("");
  //
  
  g2->cd();
  pstit = "gNeutron_16O";
  pstit += ".gif";
  g2->Print(pstit.Data());
  ////
  
  ///////// n + 16O -> n + 16O
  
  TCanvas *g3 = new TCanvas("g3","Neutron Energy",40,40,700,500);
  
  g3->SetGrid();
  
  h3->SetMinimum(1.0e-06);
  h3->SetMaximum(1.0e+00);
   
  h3->GetXaxis()->CenterTitle();
  h3->GetXaxis()->SetTitleOffset(1.2);
  h3->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Neutron Energy (MeV)";
  h3->GetXaxis()->SetTitle(tit.Data());
  
  h3->GetYaxis()->CenterTitle();
  h3->GetYaxis()->SetTitleOffset(1.0);
  h3->GetYaxis()->SetTitleSize(0.06);
  TString tit ="Neutrons /(10 #muA s 0.01 MeV)";
  h3->GetYaxis()->SetTitle(tit.Data());
  
  h3->SetLineColor(kBlue);
  h3->SetLineWidth(2.0);
  h3->SetLineStyle(2);
  h3->Draw("");
  //
  leg = new TLegend(0.55,0.70,0.85,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->AddEntry(h3,"Neutron","L");
  leg->Draw();
  leg->SetHeader("^{16}O(n,elastic)");
  leg->Draw();
  //
  
  g3->cd();
  pstit = "gNeutron_Neutron_16O";
  pstit += ".gif";
  g3->Print(pstit.Data());
  
  ////
  //
  
  TCanvas *g4 = new TCanvas("g4","Oxygen 16 Energy",40,40,700,500);
  
  g4->SetGrid();
  
  h4->SetMinimum(1.0e-06);
  h4->SetMaximum(1.0e+00);
  
  h4->GetXaxis()->CenterTitle();
  h4->GetXaxis()->SetTitleOffset(1.2);
  h4->GetXaxis()->SetTitleSize(0.06);
  TString tit = "^{16}O Energy (keV)";
  h4->GetXaxis()->SetTitle(tit.Data());
  
  h4->GetYaxis()->CenterTitle();
  h4->GetYaxis()->SetTitleOffset(1.0);
  h4->GetYaxis()->SetTitleSize(0.06);
  TString tit ="^{16}O /(10 #muA s 10 keV)";
  h4->GetYaxis()->SetTitle(tit.Data());
  
  h4->SetLineColor(kBlue);
  h4->SetLineWidth(2.0);
  h4->SetLineStyle(2);
  h4->Draw("");
  //
  
  g4->cd();
  pstit = "gNeutron_n16O";
  pstit += ".gif";
  g4->Print(pstit.Data());
  
  ////
  ////
  //
  
  TCanvas *g5 = new TCanvas("g5","Oxygen 16 Energy vs Neutron Energy",40,40,700,500);
  
  gPad->SetLogy(0);
  TStyle::SetPalette(1,0);
  g5->SetGrid();
  
  h5->GetXaxis()->CenterTitle();
  h5->GetXaxis()->SetTitleOffset(1.2);
  h5->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Neutron Energy (MeV)";
  h5->GetXaxis()->SetTitle(tit.Data());
  
  h5->GetYaxis()->CenterTitle();
  h5->GetYaxis()->SetTitleOffset(1.0);
  h5->GetYaxis()->SetTitleSize(0.06);
  TString tit ="^{16}O Energy (MeV)";
  h5->GetYaxis()->SetTitle(tit.Data());
  
  h5->SetLineColor(kRed);
  h5->SetLineWidth(2.0);
  h5->SetLineStyle(2);
  h5->Draw("CONTZ");
  
  leg = new TLegend(0.55,0.70,0.85,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->Draw();
  leg->SetHeader("^{16}O(n,elastic)");
  leg->Draw();
  
  //
  
  g5->cd();
  pstit = "gNeutron_n16On";
  pstit += ".gif";
  g5->Print(pstit.Data());
  
  ///////// n + 14N -> n + 14N
  
  TCanvas *g6 = new TCanvas("g6","Neutron Energy",40,40,700,500);
  
  g6->SetGrid();
  
  h6->SetMinimum(1.0e-06);
  h6->SetMaximum(1.0e+00);
  
  h6->GetXaxis()->CenterTitle();
  h6->GetXaxis()->SetTitleOffset(1.2);
  h6->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Neutron Energy (MeV)";
  h6->GetXaxis()->SetTitle(tit.Data());
  
  h6->GetYaxis()->CenterTitle();
  h6->GetYaxis()->SetTitleOffset(1.0);
  h6->GetYaxis()->SetTitleSize(0.06);
  TString tit ="Neutrons /(10 #muA s 0.01 MeV)";
  h6->GetYaxis()->SetTitle(tit.Data());
  
  h6->SetLineColor(kGreen);
  h6->SetLineWidth(2.0);
  h6->SetLineStyle(2);
  h6->Draw("");
  //
  leg = new TLegend(0.55,0.70,0.85,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->AddEntry(h6,"Neutron","L");
  leg->Draw();
  leg->SetHeader("^{14}N(n,elastic)");
  leg->Draw();
  //
  
  g6->cd();
  pstit = "gNeutron_Neutron_14N";
  pstit += ".gif";
  g6->Print(pstit.Data());
  
  ////
  //
  
  TCanvas *g7 = new TCanvas("g7","Nitrogen 14 Energy",40,40,700,500);
  
  g7->SetGrid();
  
  h7->SetMinimum(1.0e-06);
  h7->SetMaximum(1.0e+00);
   
  h7->GetXaxis()->CenterTitle();
  h7->GetXaxis()->SetTitleOffset(1.2);
  h7->GetXaxis()->SetTitleSize(0.06);
  TString tit = "^{14}N Energy (keV)";
  h7->GetXaxis()->SetTitle(tit.Data());
  
  h7->GetYaxis()->CenterTitle();
  h7->GetYaxis()->SetTitleOffset(1.0);
  h7->GetYaxis()->SetTitleSize(0.06);
  TString tit ="^{14}N /(10 #muA s 10 keV)";
  h7->GetYaxis()->SetTitle(tit.Data());
  
  h7->SetLineColor(kGreen);
  h7->SetLineWidth(2.0);
  h7->SetLineStyle(2);
  h7->Draw("");
  //
  
  g7->cd();
  pstit = "gNeutron_n14N";
  pstit += ".gif";
  g7->Print(pstit.Data());
  
  ////
  ////
  //
  
  TCanvas *g8 = new TCanvas("g8","Nitrogen 14 Energy vs Neutron Energy",40,40,700,500);
  
  gPad->SetLogy(0);
  TStyle::SetPalette(1,0);
  g8->SetGrid();
  
  h8->GetXaxis()->CenterTitle();
  h8->GetXaxis()->SetTitleOffset(1.2);
  h8->GetXaxis()->SetTitleSize(0.06);
  TString tit = "Neutron Energy (MeV)";
  h8->GetXaxis()->SetTitle(tit.Data());
  
  h8->GetYaxis()->CenterTitle();
  h8->GetYaxis()->SetTitleOffset(1.0);
  h8->GetYaxis()->SetTitleSize(0.06);
  TString tit ="^{14}N Energy (MeV)";
  h8->GetYaxis()->SetTitle(tit.Data());
  
  h8->SetLineColor(kRed);
  h8->SetLineWidth(2.0);
  h8->SetLineStyle(2);
  h8->Draw("CONTZ");
  
  leg = new TLegend(0.55,0.70,0.85,0.85);
  leg->SetFillColor(0);
  leg->SetShadowColor(0);
  leg->Draw();
  leg->SetHeader("^{14}N(n,elastic)");
  leg->Draw();
  
   //
  
  g8->cd();
  pstit = "gNeutron_n14Nn";
  pstit += ".gif";
  g8->Print(pstit.Data());
  
  ////// rootfile
  
  char file[100];
  sprintf(file,"gNeutron.root");
  cout << "ROOT file is " << file << endl;
   
  TFile myfile(file,"RECREATE");
  
  h1->Write();
  h2->Write();
  h3->Write();
  h4->Write();
  h5->Write();
  h6->Write();
  h7->Write();
  h8->Write();
  
  myfile.Write();
}
